%%%%%%%%%%%%%%%%%%%%%% cp10_4.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%	     Comprehensive Problem CP10.4
%	does observer-controller design for
%	flexible stick balancer, using both outputs
%		y1 = theta, y2 = x
%	states are: theta, th_dot, x, v, z, z_dot
%	Has 2 reference inputs: theta_des & x_des
%----------------------------------------------------
disp('CP10.4: Observer-controller design for')
disp('.....flexible stick balancer, using both outputs')
disp('.....y1 = theta, y2 = x')
clear;clf
% load flex.mat if exists, otherwise quit this M-file
if chkmat('flex') == 1, 
    load flex.mat
  else
     return
end
%---- plant matrices, 1 input, 2 outputs
Ap = A;
Bp = B;
disp('******>'),pause
Cp = C;
Dp = D;
Gp = ss(Ap,Bp,Cp,Dp)
disp('******>'),pause
%
pp = input('enter 6 desired CL eigenvalues as column ==> ')
Rpp_str = vec2str(real(pp));
Ipp_str = vec2str(imag(pp));
%%pp = [-1; -2; -3; -4; -5; -6]
F = place(Ap,Bp,pp)
disp('******>'),pause
L = place(Ap',Cp',2*pp)'
disp('******>'),pause
Aoc = Ap-Bp*F-L*Cp	% observer-controller (6 x 6)
Boc = L			%     (6x2)
disp('******>'),pause
Coc = -F		%     (1x6)
Doc = zeros(1,2)	%     (1x2)
Goc = ss(Aoc,Boc,Coc,Doc);
disp('******>'),pause
disp('Poles & zeros of observer-controller are:')
pOC=eig(Aoc)		% obsvr-ctrlr poles
zOC=tzero(Goc)	% obsv-ctrlr zeros
disp('******>'),pause
%--- series connection
Gocp = Goc*Gp;
[Aocp,Bocp,Cocp,Docp] = ssdata(Gocp); % 1 input,2 outputs
%---- unity fdbk connection using both theta & cart psn
%     positive sign at SJ
Gcl = feedback(Gocp,1);
[Acl,Bcl,Ccl,Dcl] = ssdata(Gcl);
%======= unit step in desired cart psn =======
t=[0:0.05:10]';
x_ref = -ones(size(t));
for i = find(t<1)',
   x_ref(i) = 0;
end
theta_ref = 0*t;
ref = [theta_ref x_ref];
[y,x] = lsim(Gcl,ref,t);
subplot(2,1,1)
plot(t,y(:,1));grid
title('CP10.4: flexible stick; obsv-ctrlr using theta & x')
xlabel('time (secs)')
ylabel('stick angle (deg)')
text('Units','norm','Pos',[0.4 0.50],'Str','Controller poles')
text('Units','norm','Pos',[0.4 0.40],'Str',...
				['real parts = ' Rpp_str])
text('Units','norm','Pos',[0.4 0.30],'Str',...
				['imag parts = ' Ipp_str])
subplot(2,1,2)
plot(t,y(:,2),t,-x_ref,'--');grid
ylabel('cart psn (ft)')
xlabel('time (secs)')
text('Units','norm','Pos',[0.4 0.80],'Str',...
		'Observer poles are 2 * controller poles')
keyboard
%====== IC response: 1 deg on stick, rest = 0 =========
%.......stick angle is state #7
x0 = [zeros(6,1); 1; zeros(5,1)];	
[y,x] = lsim(Gcl,[0*t 0*t],t,x0);
subplot(2,1,1)
plot(t,y(:,1));grid
title('CP10.4: rigid stick; initial stick angle, x_ref = 0')
xlabel('time (secs)')
ylabel('stick angle (deg)')
text('Units','norm','Pos',[0.5 0.90],'Str','Controller poles')
text('Units','norm','Pos',[0.5 0.75],'Str',...
			['real parts = ' Rpp_str])
text('Units','norm','Pos',[0.5 0.60],'Str',...
			['imag parts = ' Ipp_str])
subplot(2,1,2)
plot(t,y(:,2));grid
ylabel('cart psn (ft)')
xlabel('time (secs)')
text('Units','norm','Pos',[0.4 0.80],'Str',...
		'Observer poles are 2 * controller poles')
keyboard
disp('end of Comprehensive Problem 10.4')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
