%%%%%%%%%%%%%%%%%%%%% wif10_6.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%  This MATLAB file runs the What-if problem after 
% 	 Example 10.6 (Observer-controller design) 
%----------------------------------------------------
clear,clf
  disp('What-if following ')
  disp('Example 10.6: Observer-controller design')
  disp(' ')
  A = [-2 -2.5 -0.5; 1 0 0; 0 1 0];       % state matrix
  B = [ 1; 0; 0];                         % input matrix set to zero
  C = [1 4 3.5];                            % output matrix
  D = 0;                                  % throughput matrix
  G = ss(A,B,C,D)
  disp('*****>');pause
  p_s = [ -1.27 -3.46 -5.04 ]
  K = place(A,B,p_s)                     % control gain
  disp('*****>');pause
  p_o = [ -1.29 -2.81 -12.22 ]
  L = place(A',C',p_o)'                  % estimator gain
  disp('*****>');pause
  A_f = A-B*K-L*C                        % controller system matrix
  G_f = ss(A_f,-L,-K,0);
  f_poles = eig(A_f)                     % controller poles
  f_zeros = tzero(G_f)      		       % controller zeros
  disp('*****>');pause
  bode(G_f)       		                 % frequency response plot
  title('WIF10.6: Frequency response of controller') 
  keyboard
  G_g = G*G_f;
  [Ag,Bg,Cg,Dg]= ssdata(G_g); 				% cascade system
  G_cl = feedback(G_g,1);
  [Acl,Bcl,Ccl,Dcl] = ssdata(G_cl)			 % closed-loop system
  cl_loop_poles = eig(Acl)               	% closed-loop system poles
  disp('*****>');pause
  lfg = dcgain(G_cl)
  N = 1/lfg
  G_lN = G_cl*ss([],[],[],N);
  [AclN,BclN,CclN,DclN] = ssdata(G_lN)
  z0 = [0 0 0 1 -0.75 0.4]'
  t = [0:0.02:4]';
  u = 0*t;
  [y,t,z] = lsim(G_lN,u,t,z0);
  %plot(t,x(:,1:3),t,x(:,4:6),'--'), grid
  plot(t,z(:,1:3),t,z(:,4:6),'--'), grid
  title('WIF10.6: Initial condition response')
  keyboard
  step(G_lN,4), grid
  title('WIF10.6: Step input response')
  [ys,x,t] = step(G_lN);
  [Mo,tp,tr,ts,ess] = tstats(t,ys,1.0)
  keyboard
  disp('Note that for these choices of full-state feedback poles')
  disp('... and observer poles, the settling time is faster.')
  disp('... The poles were selected based on designs using the ')
  disp('... symmetrical root-locus technique, which can be found ')
  disp('... in the textbook by Franklin, Powell and Emami-Naeini.')
  disp('end of What If 10.6')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
