%%%%%%%%%%%%%%%%%%%%%% cp8_3b.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/14/99 HS
%----------------------------------------------------
%	     		  Comprehensive Problem CP8.3b
%  State-space satellite model, nonzero wheel speed
%	PID controller with pole for KD term
%  Runs one step response for specific set of gains
%----------------------------------------------------
disp('CP8.3B: SS model of satellite (NONzero initial wheel speed)')
clf	% do not clear workspace -- need some values
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
%
Ap = A; Bp = B; Cp = C; Dp = D;   % plant model matrices
Gpss = ss(Ap,Bp,Cp,Dp)
omega0 = input('Enter initial wheel speed in rpm ==> ')
disp('Use gains from CP8.2, or try ')
disp('........KP = -0.001 and KD = 20')
disp('Start with KI = 0, then try KI = 0.005')
KP = input('enter controller gain KP ==> ');
KI = input('enter integral gain KI ==> ');
KD = input('enter derivative gain KD ==> ');
T = 0.05*KD;	% pole for deriv term
T_str = num2str(T);
KP_str=num2str(KP);
KI_str=num2str(KI);
KD_str=num2str(KD);
omega0_str = num2str(omega0);
T = 0.05*KD;	% add pole to avoid improper Gc(s)
T_str = num2str(T);
numGc = KP*[(KD+T) (1+(KI*T)) KI];    % PID numerator
if isempty(numGc) == 1, 
   disp('Must enter numerical values for KI & KD')
   disp('....because controller numerator is empty')
   numGc
   break
end
denGc = [T 1 0];     % denominator "  " 
Gc = tf(numGc,denGc)
Gcss = ss(Gc)
[Ac,Bc,Cc,Dc] = ssdata(Gcss);   % SS controller
%--- series combination of controller & satellite
G_ol_ss = Gpss*Gcss
[Aol,Bol,Col,Dol] = ssdata(Gcss);
%
%------- closed-loop step response to reference input
%   feed back only 1st output (theta)
%--- run with zero input to allow to settle transients
t = [0:1:600]';
u = ones(size(t));	% input is unit step starting at 200 sec
for i=1:max(find(t<200)), u(i)=0;end
outputs = 1;	% used for cloop cmd
inputs  = 1;	%   "   "    "    "
G_cl_ss = feedback(G_ol_ss,1,inputs,outputs)
[Acl,Bcl,Ccl,Dcl] = ssdata(G_cl_ss);
%
%----- run zero-input response for specified initial
%	wheel speed to use final state as x0 for final run
u = zeros(size(t));
[nstates,dummy] = size(Acl);  % number of states in CL system
%x0 = [zeros(1,nstates-1) (2*pi/60)*omega0]';   % start with IC on omega
x0 = [0 0 (2*pi/60)*omega0 0 0]';   % start with IC on omega
[ys,t,x] = lsim(G_cl_ss,u,t,x0);
%----- set  x(0) = final state 
[nn,mm] = size(x);
x0 = x(nn,:);
keyboard
%
%----- CL step response to reference input
u = ones(size(t));  % input is unit step starting at 200 sec

for i=1:max(find(t<200)), u(i)=0;end
ys = lsim(G_cl_ss,u,t,x0);  
subplot(2,1,1)
plot(t,ys(:,1),t,u,'--');grid
title('CP8.3b: CL sat; ref step, PID, NONZERO wheel speed')
xlabel('Time (s)'),ylabel('theta (deg)')
subplot(2,1,2)
plot(t,ys(:,2));grid
xlabel('Time (s)'),ylabel('wheel speed (rpm)')
text('Units','Norm','Pos',[0.5,0.55],...
	'Str',['initial wheel speed = ' omega0_str ' rpm'])
text('Units','Norm','Pos',[0.5,0.40],'Str',['KP = ' KP_str])
text('Units','Norm','Pos',[0.5,0.30],'Str',['KI = ' KI_str])
text('Units','Norm','Pos',[0.5,0.20],'String',['KD = ' KD_str])
text('Units','Norm','Pos',[0.5,0.10],'Str',['T = ' T_str])
%
disp('measure step response performance of theta = y(1)')
disp(' ')
if ys(190) > 0.1
  disp('***** In order for the function ''tstats'' to work properly')
  disp('***** the values of initial wheel speed and KI must be')
  disp('***** such that the value of theta prior to the step input')
  disp('***** is less than 0.10 deg. As long as you have KI > 0, ')
  disp('***** this should not be a problem. If you do not satisfy ')
  disp('***** this condition you will get the messages: ')
  disp('*****     ???  Index exceeds matrix dimensions.')
  disp('*****   ==>  t10 = t(i10) -delt*(y(i10)-0.10*ref)...')
end
%
theta = ys(:,1);
%[Mo,tp,tr,ts,ess] = tstats(t,theta,1.0)
%
keyboard
disp('end of Comprehensive Problem 8.3b')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

