%%%%%%%%%%%%%%%%%%%%% wif8_3d.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/21/99 HS
%----------------------------------------------------
%  This MATLAB file runs the What-if problem after 
%  						Example 8.3
%----------------------------------------------------
clear,clf
disp('Wif 8.3d: PID control with ramp inputs')
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
numGp = Kproc;
denGp = conv([tauP1 1],[tauP2 1]);
H  = tf(1,[tau_sen 1]);
KP = 7;
KI = 0.5;
KD = 0.4;
KP_str=num2str(KP);
KI_str=num2str(KI);
KD_str=num2str(KD);
numKc = KP*[KD 1 KI];   	% numerator of PID controller
numGK = conv(numKc,numGp);
disp('PID controller transfer functon:')
GK = tf(numGK,[denGp 0])
%----- closed-loop ramp response to reference input
T_ref = feedback(GK,H,-1);
t = [0:0.01:3];
u = t;
y_ref = lsim(T_ref,u,t);

%=========== repeat for disturbance input ===========
disp('Plant transfer function:')
Gp = tf(numGp,denGp)
HK = tf(-numKc,[tau_sen 1 0]);
%-------- CL ramp response to disturbance input
%--- use +1 for feedback gain because neg sign is in HK(s)
disp('CL transfer function:')
T_dist = feedback(Gp,HK,+1)
u = t;
y_dist = lsim(T_dist,u,t);

%-------- plot both responses
disp('Plot CL response to ramp reference and disturbance inputs')
disp('*******>');pause
plot(t,y_ref,'--',t,y_dist,'-.',t,u)
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
%title('Wif 8.3d: CL ref & dist ramps responses, PID control')
text(0.82,0.50,['KP = ' KP_str])
text(0.82,0.40,['KI = ' KI_str])
text(0.82,0.30,['KD = ' KD_str])
text(0.7,0.63,'reference')
text(0.7,0.1,'disturbance')
axis([0 1 0 1])
keyboard
disp('end of What If 8.3d')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%