%%%%%%%%%%%%%%%%%%%%%% cp9_1a.m %%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/15/99 HS
%----------------------------------------------------
%  M-file for designing lag controller for the power 
%   system example for Comprehensive Problem CP9.1a
%----------------------------------------------------
clear,clf
disp('Comprehensive Problem CP9.1a')

epow    % read data

C = C(1,:);
cflag = input('enter 1 to add more damping to system ==> ')
if cflag == 1
  A(2,2) = 10*A(2,2);
end

disp('Gain and phase margins, from V_field to V_term')
Gpss = ss(A,B,C,0);
Gp = tf(Gpss);
margin(Gp)
disp('*****>'), pause

disp('Lag control via Franklin''s algorithm')

pm_des = input('enter desired phase margin in degrees ==>')
pm_act = pm_des + 10;  % add 10 deg to desired margin for lag design
w = logspace(-2,3,100)';
[mag,phase] = bode(Gp,w);
phase_0 = phase - pm_act + 180;    % = 0 if matches pm_act
[ph,i] = min(abs(phase_0));        % best match
Klag = 1/mag(i)                    % proportional gain
%---------- open-loop transfer function
KGp = Klag*Gp;
margin(KGp)
[mag_ratio_K,ph_K,w] = bode(KGp);
mag_db_K = 20*log10(mag_ratio_K);
disp('*********>');pause

ess_des = input('Enter desired steady-state error ==> ')
Kp_des = (1/ess_des) - 1
lfg = dcgain(KGp)
Alag = Kp_des/lfg
disp('*********>');pause
disp('Specify value for zero corner freq in interval ')
disp('      wpm/10 < zero corner freq < wpm/2')
disp('   where wpm is the phase-margin frequency without lag')
w_corner = input('Enter corner freq for lag zero ==> ')
Zlag = -w_corner
%------------- build the lag then use
%		forward transfer function is Gc * Gp
numGc = Klag*[1 -Zlag];
denGc = [1 -Zlag/Alag];
Gc = tf(numGc,denGc);
Gcp = Gc*Gp;
[gm,pm,wgm,wpm] = margin(Gcp);
disp('gain margin in dB & omega_gm are:')
disp([20*log10(gm)  wgm])
disp('phase margin in degrees & omega_pm are:')
disp([pm wpm])
disp('*********>');pause
%---- do the plot
margin(Gcp)
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(Gcp)

%------------- closed-loop step response to reference input
Gcp_CL = feedback(Gcp,1);
z=tzero(Gcp_CL)
p=pole(Gcp_CL)
t = [0:0.02:10];
step(Gcp_CL,t)
grid;title('CP9.1a: Closed loop step response')
keyboard
[mag_ratio_lag,ph_lag] = bode(Gcp_CL,w);
mag_db_lag = 20*log10(mag_ratio_lag);
%--------- now plot both sets
zdb = 0*w;
subplot(2,1,1)
semilogx(w,mag_db_lag(:),'-',w,mag_db_K(:),'--',w,zdb,':');
grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
title('CP9.1a: Lag controller (solid); Proportional controller (- -)')
subplot(212)
ph180 = 0*w-180;
semilogx(w,ph_lag(:),'-',w,ph_K(:),'--',w,ph180,':');
grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')

disp('end of Comprehensive Problem 9.1a')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
