%%%%%%%%%%%%%%%%%%%%%% ex9_4.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
clear,clf
disp('Lead-lag control via frequency response')
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]))
H  = tf(1,[tau_sen 1])
%---------- open-loop transfer function
GpH = Gp*H

disp('Part (a) determine Kldlg & check phase margin')
Kp_reqd = 400       % will give ess = 1/401
plant_lfg = dcgain(GpH)
Kldlg = 400/plant_lfg
Kldlg_str = num2str(Kldlg);
disp('*********>');pause
[km,pm,wkm,wpm] = margin(Kldlg*GpH);
pm
disp('*********>');pause
disp('Part (b) insert lead & find alpha & zero values')
w = logspace(-1,2,100)';
[mag,ph] = bode(Kldlg*GpH,w);
mag = reshape(mag,100,1);
ph  = reshape(ph, 100,1);
disp('using pm target = 35 deg')
pm_target = 35
del_pm = pm_target - pm        % max phase angle in deg 
Alead = (1+sin(del_pm*pi/180))/(1-sin(del_pm*pi/180))% (9.2)
Alead_str = num2str(Alead);
wc = interp1(mag,w,1/sqrt(Alead)) % use (9.4) to find wc
Zlead = -wc/sqrt(Alead)            % using (9.3)
Zlead_str = num2str(Zlead);
disp('*********>');pause
Gld = tf(Kldlg*Alead*[1  -Zlead],[1  -Alead*Zlead]);   % use (9.1) for lead TF
%---- OL transfer function of lead, plant, & sensor
[km,pm,wkm,wpm] = margin(GpH*Gld);  % new pm value
pm
disp('*********>');pause
disp('Part (c) now do lag part')
[mag_ld,ph_ld] = bode(GpH*Gld,w);
mag_ld = reshape(mag_ld,100,1);
ph_ld  = reshape(ph_ld, 100,1);
disp('using desired phase at new w_c = -124 deg')
ph_at_wc = -124;
wc = interp1(ph_ld,w,ph_at_wc)
disp('*********>');pause
mag_ld_wc = bode(GpH*Gld,wc)  % only one freq
Alag = mag_ld_wc
Alag_str = num2str(Alag);
Zlag = -wc/10
Zlag_str = num2str(Zlag);
disp('*********>');pause
Glg = tf([-1/Zlag  1],[-Alag/Zlag  1]) % use Kuo's form for lag
%----- complete lead-lag controller
Gc = Gld*Glg
disp('*********>');pause
%---- open-loop transfer function of compensated system
GcGpH = Gc*GpH
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(GcGpH)
[km,pm,wkm,wpm] = margin(GcGpH); % want pm = 50
pm
disp('*********>');pause

%==========  OL frequency responses for Fig. 9.10 =====
w = logspace(-3,3,100);
ph180 = -180*ones(length(w));
%---- first, with gain only
[mag_K,ph_K] = bode(Kldlg*GpH,w);
mag_K = reshape(mag_K,100,1);
ph_K  = reshape(ph_K, 100,1);
mag_db_K = 20*log10(mag_K);
%-------- then with final lead design
[mag_lead,ph_lead] = bode(GcGpH,w);
mag_lead = reshape(mag_lead,100,1);
ph_lead  = reshape(ph_lead, 100,1);
mag_db_lead = 20*log10(mag_lead);
%--------- now plot both sets
subplot(211)
semilogx(w,mag_db_lead,'-',w,mag_db_K,'--');grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
text(1.5,-30,'Lead-lag')
text(20,10,'Proportional')
title('Ex 9.4: OL freq response per Fig. 9.10')
subplot(212)
semilogx(w,ph_lead,'-',w,ph_K,'--',w,ph180,'-');grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
text(20,-130,'Lead-lag')
text(0.5,-230,'Proportional')
keyboard
subplot(111)	% reset plot window

%==========  CL step responses for Fig. 9.11 =====
%---- build CL system for ref input with sensor in fdbk path
T_ref = feedback(Gp*Gc,H,-1)
disp('*********>');pause
t = [0:0.01:4];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Gc*H,+1)
disp('*********>');pause
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 9.4: CL ref & dist step responses Fig. 9.11')
text(2.0,0.80,['Kldlg = ' Kldlg_str])
text(2.0,0.70,['Zlead = ' Zlead_str])
text(2.0,0.60,['Alead = ' Alead_str])
text(2.0,0.50,['Zlag = ' Zlag_str])
text(2.0,0.40,['Alag = ' Alag_str])
axis([0 4 0 1.2])
keyboard

disp('Part (d) determine bandwidth of CL system')
%.... using narrow freq interval
ww = logspace(0,1,100);
[mag_CL,ph_CL] = bode(T_ref,ww);
mag_CL = reshape(mag_CL,100,1);
mag_CL_db = 20*log10(mag_CL);
lfg = dcgain(T_ref)
lfg_db = 20*log10(lfg)
bw_CL = bwcalc(mag_CL_db,ww,lfg_db)
%--- plot mag with line at mag_CL_db(1)-3 dB
A = lfg_db - 3;
Aline = A*ones(size(ww));
semilogx(ww,mag_CL_db,ww,Aline);grid
title('Ex 9.4: details of closed-loop bandwidth calc')
xlabel('frequency (rad/s)')
ylabel('Magnitude (dB)')
disp('end of Example Problem 9.4')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
