%%%%%%%%%%%%%%%% fig9_11.m %%%%%%%%%%%%%%%
% 	Jan. 99
%  last revision:  1/8/99  JHC
%----------------------------------------
clear;clf
set(gcf,'DefaultAxesFontSize',16)
set(gcf,'DefaultTextFontSize',16)
disp('fig 9.11: Lead-Lag control via Kuo''s algorithm ')
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]))
H  = tf(1,[tau_sen 1])
GpH = Gp*H
%-------- final lead-lag design
Kldlg = 100; Zlead = -6.433; Alead = 25.66;
Zlag = -0.3379; Alag = 33.21;
%----------
w = logspace(-3,3,100);
ph180 = -180*ones(length(w));
%----- first do gain-only
[mag_K,ph_K] = bode(Kldlg*GpH,w);
mag_K = reshape(mag_K,100,1);
ph_K  = reshape(ph_K, 100,1);
mag_db_K = 20*log10(mag_K);
%
%-------- do calcs with final lead-lag design
Glead = tf(Alead*[1  -Zlead],[1  -Alead*Zlead]);
Glag  = tf([-1/Zlag  1],[-Alag/Zlag  1]);
Gc    = Kldlg*Glead*Glag

Kldlg_str=num2str(Kldlg);
Zlead_str = num2str(Zlead);
Alead_str = num2str(Alead);
Zlag_str = num2str(Zlag);
Alag_str = num2str(Alag);

%==========  CL step responses for Fig. 9.11 =====
%---- build CL system for ref input with sensor in fdbk path
T_ref = feedback(Gp*Gc,H,-1)
t = [0:0.01:4];
y_ref = step(T_ref,t);
%----
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Gc*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
%title('fig 9.11: CL ref & dist step resp, lead-lag, Ex. 9.4')
text(2.5,0.75,['Kldlg= ' Kldlg_str])
text(2.5,0.68,['Zlead = ' Zlead_str])
text(2.5,0.61,['Alead = ' Alead_str])
text(2.5,0.54,['Zlag = ' Zlag_str])
text(2.5,0.47,['Alag = ' Alag_str])
text(2.5,1.03,'reference')
text(2.5,0.20,'disturbance')
axis([0 4 0 1.2])
print -deps fig9_11.eps
%-- reset fonts to defaults
set(gcf,'DefaultAxesFontSize',get(0,'DefaultAxesFontSize'));
set(gcf,'DefaultTextFontSize',get(0,'DefaultTextFontSize'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%